"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const c_sharp_objects_1 = require("./c-sharp-objects");
class CSharpParser {
    static parse(input) {
        input = CSharpParser.stripIgnored(input);
        input = CSharpParser.stripComments(input);
        const namespaceMatch = input.match(/namespace\s+([\w\.]+)/);
        const namespace = namespaceMatch ? namespaceMatch[1] : '';
        const types = [];
        let typeMatch;
        const typeRegex = /^([\t ]*)(?:public\s*|partial\s*|abstract\s*)*\s*(enum|class|struct|interface)\s+([\w\d_<>]+)(?:\s*:\s*((?:(?:[\w\d\.\<\>_]+)(?:,\s+)?)+))?\s*\{((?:.|\n|\r)*?)^\1\}/gm;
        const getNextTypeMatch = () => typeMatch = typeRegex.exec(input);
        while (getNextTypeMatch() !== null) {
            const type = typeMatch[2];
            const name = typeMatch[3];
            const inherits = CSharpParser.parseInherits(typeMatch[4]);
            const body = typeMatch[5];
            if (type === 'class' || type === 'struct') {
                types.push(CSharpParser.parseClassOrStruct(namespace, name, inherits, body));
            }
            else if (type === 'interface') {
                types.push(CSharpParser.parseInterface(namespace, name, inherits, body));
            }
            else if (type === 'enum') {
                types.push(CSharpParser.parseEnum(namespace, name, inherits, body));
            }
        }
        return types;
    }
    static stripIgnored(input) {
        const ignoredRegex = /\/\/\s*ts-generator-ignore\s*.*(enum|class|struct|interface)\s+/gm;
        return input.replace(ignoredRegex, '');
    }
    static stripComments(input) {
        const blockCommentRegex = /\/\*([\s\S]*?)\*\//gm;
        const lineCommentRegex = /\/\/(.*)/g;
        return input
            .replace(blockCommentRegex, '')
            .split('\n')
            .map(line => line.replace(lineCommentRegex, ''))
            .join('\n');
    }
    static parseInherits(inheritsStr) {
        const inherits = [];
        if (inheritsStr) {
            let inheritsMatch;
            const inheritsRegex = /(?:[\w\d\.\_]+)(?:\<.+?\>)?/g;
            const getNextInheritsMatch = () => inheritsMatch = inheritsRegex.exec(inheritsStr);
            while (getNextInheritsMatch() !== null) {
                inherits.push(inheritsMatch[0]);
            }
        }
        return inherits;
    }
    static parseClassOrStruct(namespace, name, inherits, body) {
        const constructors = [];
        let constructorMatch;
        const constructorRegex = /public\s+([\w]+)\s*\(((?:.|\r|\n)*?)\)/gm;
        const getNextConstructorMatch = () => constructorMatch = constructorRegex.exec(body);
        while (getNextConstructorMatch() !== null) {
            const typeName = constructorMatch[1];
            const parameterList = constructorMatch[2];
            if (typeName === name) {
                const parameters = parameterList.length ? parameterList
                    .split(',')
                    .map(parameter => {
                    const parameterMatch = parameter.trim().match(/([^\s]+)\s+([\w\d]+)(?:\s+=\s+((?:[\w\d]+)|(?:"[\w\d ]+")))?/);
                    const parameterType = CSharpParser.parseMemberTypeName(parameterMatch[1]);
                    const parameterName = parameterMatch[2];
                    const rawDefaultParameter = parameterMatch[3];
                    const defaultValue = rawDefaultParameter && rawDefaultParameter.startsWith('"') && rawDefaultParameter.endsWith('"') ?
                        `'${rawDefaultParameter.substr(1, rawDefaultParameter.length - 2)}'` : rawDefaultParameter;
                    return new c_sharp_objects_1.CSharpParameter(parameterType, parameterName, defaultValue);
                }) : [];
                constructors.push(new c_sharp_objects_1.CSharpContructor(parameters));
            }
        }
        const properties = CSharpParser.getProperties(body, false);
        return new c_sharp_objects_1.CSharpClassOrStructOrInterface(namespace, name, inherits, constructors, properties);
    }
    static parseInterface(namespace, name, inherits, body) {
        const properties = CSharpParser.getProperties(body, true);
        return new c_sharp_objects_1.CSharpClassOrStructOrInterface(namespace, name, inherits, null, properties);
    }
    static parseEnum(namespace, name, inherits, body) {
        const entries = [];
        let entryMatch;
        const entryRegex = /^\s*([\w\d_]+)\s*=?\s*([0-9A-Fx]+)?,?\s*$/gm;
        const getNextEntryMatch = () => entryMatch = entryRegex.exec(body);
        while (getNextEntryMatch() !== null) {
            const entryName = entryMatch[1];
            const entryValue = entryMatch[2];
            const radix = /^[0-9]+$/.test(entryValue) ? 10 : 16;
            const entryValueParsed = parseInt(entryValue, radix);
            if (entryName.indexOf('[') === -1) {
                entries.push(new c_sharp_objects_1.CSharpEnumEntry(entryName, entryValueParsed));
            }
        }
        return new c_sharp_objects_1.CSharpEnum(namespace, name, inherits, entries);
    }
    static parseMemberTypeName(typeName) {
        const lastNameRegex = /\.?([\w<>\[\]\?]+)$/;
        const nullableRegex = /(?:^Nullable<([^\s]+)>$)|(?:^([^\s]+)\?$)/;
        const nullableMatch = nullableRegex.exec(typeName);
        if (nullableMatch) {
            typeName = (nullableMatch[1] || nullableMatch[2]);
        }
        const collectionRegex = /(?:List|IEnumerable)<([\w\d\.]+)>$|(?:^([\w\d\.]+)\[\]$)/;
        const collectionMatch = collectionRegex.exec(typeName);
        if (collectionMatch) {
            typeName = (collectionMatch[1] || collectionMatch[2]);
        }
        typeName = typeName.match(lastNameRegex)[1];
        return new c_sharp_objects_1.CSharpMemberType(typeName, !!nullableMatch, !!collectionMatch);
    }
    static getProperties(body, isInterface) {
        const properties = [];
        let propertyMatch;
        const propertyRegex = isInterface ? /\s*([^\s]+)\s*([\w\d]+)\s*{\s*get/gm : /public\s*([^\s]+)\s*([\w\d]+)\s*{\s*get/gm;
        const getNextPropertyMatch = () => propertyMatch = propertyRegex.exec(body);
        while (getNextPropertyMatch() !== null) {
            const typeName = propertyMatch[1];
            const propertyName = propertyMatch[2];
            const memberType = CSharpParser.parseMemberTypeName(typeName);
            properties.push(new c_sharp_objects_1.CSharpProperty(memberType, propertyName));
        }
        return properties;
    }
}
exports.CSharpParser = CSharpParser;
//# sourceMappingURL=c-sharp-parser.js.map