#! /usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const globby = require("globby");
const path = require("path");
const fs_utilities_1 = require("./fs.utilities");
const generate_schema_1 = require("./generate-schema");
const output_item_1 = require("./output-item");
const tsgen_1 = require("./tsgen");
const defaultConfigFileName = 'tsgen.json';
(function () {
    const configFilePath = getConfigFilePath();
    const workingDirectory = path.dirname(configFilePath);
    const config = require(configFilePath);
    if (config.outDir === undefined) {
        throw new Error('Invalid config: must specify an outDir.');
    }
    const sourcePaths = globby.sync(config.src, { cwd: workingDirectory })
        .map(file => path.normalize(path.join(workingDirectory, file)));
    const outputItems = generateOutput(sourcePaths, config.options);
    writeOutput(workingDirectory, config, outputItems);
})();
function getConfigFilePath() {
    let configFileName = defaultConfigFileName;
    if (process.argv.length > 2) {
        configFileName = process.argv[2];
    }
    try {
        const stats = fs.lstatSync(configFileName);
        if (!stats.isFile()) {
            throw new Error(`${configFileName} is not a file.`);
        }
    }
    catch (e) {
        throw new Error(`${configFileName} does not exist.`);
    }
    return path.normalize(path.join(process.cwd(), configFileName));
}
function generateOutput(sourcePaths, options) {
    const outputItems = [];
    for (const filePath of sourcePaths) {
        const name = path.basename(filePath, '.cs');
        const csharp = fs.readFileSync(filePath).toString();
        const typescript = tsgen_1.tsGenerator(csharp, options);
        outputItems.push(new output_item_1.OutputItem(name, typescript));
    }
    const csharps = sourcePaths.map(filePath => fs.readFileSync(filePath).toString());
    outputItems.push(new output_item_1.OutputItem('schema', generate_schema_1.generateSchema(csharps)));
    return outputItems;
}
function writeOutput(workingDirectory, config, outputItems) {
    const outputDirPath = path.normalize(path.join(workingDirectory, config.outDir));
    const generatedFilePath = path.join(outputDirPath, 'generated.ts');
    const importPaths = [...(config.imports || [])]
        .map(importPath => path.isAbsolute(importPath) ? importPath : path.normalize(path.join(workingDirectory, importPath)));
    const tsCode = outputItems.map(item => item.typescript);
    if (importPaths.length) {
        const importCode = importPaths
            .map(importPath => generateImports(importPath, outputDirPath))
            .join('\n\n');
        tsCode.unshift(importCode);
    }
    const concatenatedTypescript = tsCode
        .map(code => code.trim())
        .filter(code => code !== undefined && code.length > 0)
        .join('\n\n');
    fs_utilities_1.safeWriteFileSync(generatedFilePath, `/* tslint:disable */\n\n${concatenatedTypescript}\n`);
    writeIndex(workingDirectory, generatedFilePath, config);
}
function writeIndex(workingDirectory, generatedFilePath, config) {
    const outputDirPath = path.dirname(generatedFilePath);
    const indexFilePath = path.join(outputDirPath, 'index.ts');
    const exportPaths = [...(config.exports || []), generatedFilePath]
        .map(exportPath => path.isAbsolute(exportPath) ? exportPath : path.normalize(path.join(workingDirectory, exportPath)));
    const exportCode = exportPaths
        .map(exportPath => generateExports(exportPath, outputDirPath))
        .join('\n\n');
    fs_utilities_1.safeWriteFileSync(indexFilePath, `/* tslint:disable */\n\n${exportCode}\n`);
}
function generateImports(sourcePath, outputDirPath) {
    return generateImportsOrExports('import', sourcePath, outputDirPath);
}
function generateExports(sourcePath, outputDirPath) {
    return generateImportsOrExports('export', sourcePath, outputDirPath);
}
function generateImportsOrExports(type, sourcePath, outputDirPath) {
    const exportRegex = /export\s+(?:(?:(?:class|const|enum|interface|function)\s+([0-9A-Za-z_]+))|(?:{\s+([0-9A-Za-z_]+)\s+}))/g;
    const source = fs.readFileSync(sourcePath).toString();
    let match;
    const exportedNames = [];
    while ((match = exportRegex.exec(source)) !== null) {
        exportedNames.push(match[1] ? match[1] : match[2]);
    }
    const relativeReferencePath = path.relative(outputDirPath, sourcePath).replace(/\\/g, '/').replace(/\.ts$/, '');
    return `${type} {\n  ${exportedNames.join(',\n  ')}\n} from './${relativeReferencePath}';`;
}
//# sourceMappingURL=tsgen-cli.js.map